# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/14 3:44 下午

class DBFunction:
    def __init__(self, function_body, value):
        self.function_body = function_body
        self.value = value
